package test_1_sub_3_d;
require Exporter;
@ISA = qw ( Exporter );
@EXPORT = qw (initializeStatistic calculateStatistic);

sub initializeStatistic { return 0; }
sub calculateStatistic { return 0; }

1;
